/*
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#include "main.h"
#include "shared.h"

/* Signal handler */
static void sig_handler( int signal );

/*------------------------------------------------------------------------*/

  int
main (int argc, char *argv[])
{
  /* Command line option returned by getopt() */
  int option;

  /* New and old actions for sigaction() */
  struct sigaction sa_new, sa_old;


  /* Initialize new actions */
  sa_new.sa_handler = sig_handler;
  sigemptyset( &sa_new.sa_mask );
  sa_new.sa_flags = 0;

  /* Register function to handle signals */
  sigaction( SIGINT,  &sa_new, &sa_old );
  sigaction( SIGSEGV, &sa_new, 0 );
  sigaction( SIGFPE,  &sa_new, 0 );
  sigaction( SIGTERM, &sa_new, 0 );
  sigaction( SIGABRT, &sa_new, 0 );

  /* Initialize GTK+ */
  gtk_init (&argc, &argv);

  /*** Process command line options ***/
  while( (option = getopt(argc, argv, "hv") ) != -1 )
    switch( option )
    {
      case 'h': /* Print usage and exit */
        Usage();
        exit( 0 );

      case 'v': /* Print version and exit */
        puts( PACKAGE_STRING );
        exit( 0 );

      default: /* Print usage and exit */
        Usage();
        exit( -1 );

    } /* End of switch( option ) */

  /* Create file path to xpsk31 glade file */
  Strlcpy( rc_data.xpsk31_glade,
      getenv("HOME"), sizeof(rc_data.xpsk31_glade) );
  Strlcat( rc_data.xpsk31_glade,
      "/xpsk31/xpsk31.glade", sizeof(rc_data.xpsk31_glade) );

  /* Check for the Glade config file */
  FILE *fp = fopen( rc_data.xpsk31_glade, "r" );
  if( fp == NULL )
  {
    fprintf( stderr, "xpsk31: cannot open xpsk31 Glade GUI Description file.\n" );
    perror( rc_data.xpsk31_glade );
    fprintf( stderr, "xpsk31: trying to create xpsk31 config directory "
                     "from the installation prefix file tree.\n" );

    /* Find the binary's path (location in file system) */
    char exe_path[256], file_path[288];
    
    /* Read the file path to xpsk31 executable */
    size_t len = sizeof( exe_path );
    int bytes = (int)readlink( "/proc/self/exe", exe_path, len );
    if( bytes <= 0 )
    {
      fprintf( stderr, "xpsk31: cannot read xpsk31 binary's location.\n" );
      perror( "/proc/self/exe" );
      exit( -1 );
    }

    /* Remove "/bin/xpsk31" from the path with room for termination */
    bytes -= sizeof( "/bin/xpsk31" ) - 1;
    if( bytes < 1 )
    {
      fprintf( stderr, "xpsk31: cannot create file path to examples/xpsk31.\n" );
      exit( -1 );
    }

    /* Create file path to xpsk31 examples directory */
    exe_path[bytes] = '\0';
    Strlcpy( file_path, exe_path, sizeof(file_path) );
    Strlcat( file_path, "/share/examples/xpsk31", sizeof(file_path) );
    fprintf( stderr, "xpsk31: creating xpsk31 config directory from: %s\n", file_path );

    /* Create system command to copy examples/xpsk31 to ~/.xpsk31 */
    char syscmnd[512];
    Strlcpy( syscmnd, "cp -r ", sizeof(syscmnd) );
    Strlcat( syscmnd, file_path, sizeof(syscmnd) );
    Strlcat( syscmnd, " ", sizeof(syscmnd) );
    Strlcat( syscmnd, getenv("HOME"), sizeof(syscmnd) );
    Strlcat( syscmnd, "/xpsk31",   sizeof(syscmnd) );
    int ret = system( syscmnd );
    if( ret == -1 )
    {
      fprintf( stderr,"xpsk31: cannot create xpsk31's working directory.\n" );
      perror( file_path );
      exit( -1 );
    }
  } /* if( fp == NULL ) */
  else fclose( fp );

  /* Create main window and menu */
  main_window = create_main_window( &main_window_builder );
  gtk_window_set_title( GTK_WINDOW(main_window), PACKAGE_STRING );
  popup_menu = create_popup_menu( &popup_menu_builder );

  /* Get Rx text buffer and scroller */
  rx_text_buffer = gtk_text_view_get_buffer
    ( GTK_TEXT_VIEW(Builder_Get_Object(main_window_builder, "rx_textview")) );
  rx_scrolledwindow = Builder_Get_Object( main_window_builder, "rx_scrolledwindow" );

  /* Get Tx text buffer and scroller */
  tx_text_buffer = gtk_text_view_get_buffer
    ( GTK_TEXT_VIEW(Builder_Get_Object(main_window_builder, "tx_textview")) );
  tx_scrolledwindow = Builder_Get_Object( main_window_builder, "tx_scrolledwindow" );

  /* Get scope widget */
  scope = Builder_Get_Object( main_window_builder, "scope" );
  scope_label = Builder_Get_Object( main_window_builder, "scope_label" );

  /* Set labels to Xmit and Rcve buttons */
  {
    GtkWidget *button;

    button = Builder_Get_Object( main_window_builder, "xmit" );
    xmit_status = gtk_label_new( _("TRANSMIT") );
    gtk_widget_show( xmit_status );
    gtk_container_add( GTK_CONTAINER(button), xmit_status );
    g_object_set_data_full( G_OBJECT(main_window), "xmit_status",
        g_object_ref(xmit_status), (GDestroyNotify)g_object_unref );

    button = Builder_Get_Object( main_window_builder, "rcve" );
    rcve_status = gtk_label_new( _("RECEIVE") );
    gtk_widget_show( rcve_status );
    gtk_container_add( GTK_CONTAINER(button), rcve_status );
    g_object_set_data_full( G_OBJECT(main_window), "rcve_status",
        g_object_ref( rcve_status ), (GDestroyNotify)g_object_unref );
  }

  /* Show main window */
  gtk_widget_show (main_window);

  /* Get waterfall size */
  GtkAllocation alloc;
  gtk_widget_get_allocation( scope, &alloc );
  if( !Create_Scope_Pixbuf(alloc.width, alloc.height) )
    Error_Dialog( _("Failed to create pixbuf for waterfall"), QUIT );

  /* Load runtime config file, abort on error */
  g_idle_add( Load_Config, NULL );

  /* Default mode is BPSK */
  SetFlag( MODE_BPSK );
  SetFlag( ENABLE_AFC );

  gtk_main ();

  return 0;
}

/*-------------------------------------------------------------------*/

/*  Usage()
 *
 *  Prints usage information
 */

  void
Usage( void )
{
  fprintf( stderr, "%s\n",
      _("Usage: xpsk31 [-hv]") );

  fprintf( stderr, "%s\n",
      _("       -h: Print this usage information and exit.") );

  fprintf( stderr, "%s\n",
      _("       -v: Print version number and exit.") );

} /* End of Usage() */

/*------------------------------------------------------------------------*/

/*  sig_handler()
 *
 *  Signal Action Handler function
 */

static void sig_handler( int signal )
{
  /* Cleanup and abort */
  Cleanup();

  fprintf( stderr, "\n" );
  switch( signal )
  {
    case SIGINT :
      fprintf( stderr, "%s\n",  _("xpsk31: Exiting via User Interrupt") );
      exit( signal );

    case SIGSEGV :
      fprintf( stderr, "%s\n",  _("xpsk31: Segmentation Fault") );
      exit( signal );

    case SIGFPE :
      fprintf( stderr, "%s\n",  _("xpsk31: Floating Point Exception") );
      exit( signal );

    case SIGTERM :
      fprintf( stderr, "%s\n",  _("xpsk31: Termination Request received") );
      exit( signal );

    case SIGABRT :
      fprintf( stderr, "%s\n",  _("xpsk31: Abort Signal received") );
      exit( signal );

  } /* switch( signal ) */

} /* End of sig_handler() */

/*------------------------------------------------------------------------*/

